classdef sepexp
    %SEPEXP Class for separator testing experiment
    properties
        dp          % dispersed phase name, e.g. water
        cp          % continuous phase name, e.g. Galden HT270
        bp_id       % ID of back pressure (outlet) tubing (microns)
        bp_length   % length of BP tubing (cm)
    end

    properties (SetAccess=private)
        dpdensity                       % (g/mL)
        tfr = 200:100:2000              % total flow rate (uL/min)
        dpcp = 1                        % ratio of DP:CP
        mass_initial                    % DP outlet (g)
        mass_final                      % (g)
        exp_order                       % order TFRs were carried out
        time_stab = 2;                  % stabilisation time (min)
        time_collect = 4;               % collection time (min)

        % pause can be unreliable, so use tic-toc and record actual times
        time_collect_elapsed            % (min)
        time_stab_elapsed               % (min)
    end
     
    properties (Dependent = true)
        cpfr % flow rate of CP (uL/min)
        dpfr % flow rate of DP

        % Measurements
        dp_fr_out % collection rate at outlet (uL/min)
        dp_vol_out % volume collected at outlet (uL)
    end
    
    methods
        %% Constructor
        function obj = sepexp(dp, cp, bp_id, bp_length)
            if nargin ~= 4
                error('Must specifiy dp, cp, bp_id, bp_length')
            end
                
            % check DP is water or toluene   
            if ~(strcmpi(dp, 'water') || strcmpi(dp, 'toluene'))
                error('DP must be water or toluene')
            else
                obj.dp = dp;
            end

            % set dpdensity based on dp
            switch lower(dp)
                case 'water'
                    obj.dpdensity = 1;
                case 'toluene'
                    obj.dpdensity = 0.87;
            end
                
            % check CP is toluene or HT230
            if ~(strcmpi(cp, 'HT230') || strcmpi(cp, 'toluene'))
                error('CP must be HT230 or toluene')
            else
                obj.cp = cp;
            end
            
            % set bp tubing info
            obj.bp_id = bp_id;
            obj.bp_length = bp_length;
            
            % preallocate
            obj.mass_initial = obj.tfr .* NaN;
            obj.mass_final = obj.tfr .* NaN;
        end
        
        %% Run a single experiment
        function obj = runone(obj)             
                % index of exps not run yet
                notDone = find(isnan(obj.mass_initial) == 1);

                % pick one from notDone at random
                i = notDone(randi([1 length(notDone)]));

                % put it in exp_order to record order of measurements
                obj.exp_order = [obj.exp_order i];    

                % set FRs and start pumps
                syrris('ch1FR',obj.cpfr(i),'ch2FR',obj.dpfr(i),...
                    'ch1infuse',true,'ch2infuse',true);

                % print estimated time the pause will end
                eta = @(min) datestr(now + (min/(24*60)),'HH:MM');
                
                % Wait for stabilisation. Use tic-toc.
                fprintf('Stabilising. Estimated completion time: %s\n',...
                    eta(obj.time_stab))
                tic
                pause(obj.time_stab*60)
                obj.time_stab_elapsed = toc / 60; % minutes

                % stop, weigh to get initial mass
                syrris('ch1infuse',false,'ch2infuse',false);
                obj.mass_initial(i) = weigh();

                % re-start
                syrris('ch1infuse',true,'ch2infuse',true);

                % wait for collection
                fprintf('Collecting. Estimated completion time: %s\n',...
                    eta(obj.time_collect))
                tic
                pause(obj.time_collect*60)
                obj.time_collect_elapsed = toc / 60; % minutes

                % stop, weigh to get final mass
                syrris('ch1infuse',false,'ch2infuse',false);
                obj.mass_final(i) = weigh();
        end
        
        %% Run all experiments
        function obj = runall(obj)
            while sum(isnan(obj.mass_initial)) > 0
                expNum = length(obj.mass_initial) - ...
                    sum(isnan(obj.mass_initial));
                fprintf('-- Experiment %i of %i --\n', expNum,...
                    length(obj.mass_initial));
                obj = obj.runone();
            end
        end
                
        %% Plot results
        function h = plot(varargin)
            x = [];
            y = [];
            y_err = [];
            name = cell(nargin, 1);
            
            for i = 1:nargin
                obj = varargin{i};
                name{i} = sprintf('%i um', obj.bp_id);

                % Mass of DP injected
                % (uL/min / 1000) * min * g/mL = mL * g/mL = g
                dp_mass_in = (obj.dpfr./1000) .* obj.dpdensity ... 
                    .* obj.time_collect_elapsed;

                % Mass of liquid collected at the outlet
                mass_out = obj.mass_final - obj.mass_initial;
                
                % Volume of DP collected at the outlet
                dp_vol = (mass_out <= dp_mass_in) .* (mass_out / obj.dpdensity);
                dp_vol = dp_vol + (mass_out > dp_mass_in) .* (dp_mass_in / obj.dpdensity);
                
                % Volume of CP collected at the outlet
                switch lower(obj.cp)
                    case 'ht230'
                        cpdensity = 1.82;
                    case 'toluene'
                        cpdensity = 0.87;
                end
                
                cp_vol = (mass_out > dp_mass_in) .* ((mass_out - dp_mass_in) ./ cpdensity);
                
                % Volumetric collection rate at the outlet
                outlet = (dp_vol + cp_vol) .* (1000 / obj.time_collect_elapsed);
                
                % Add to matrices for plotting
                x(:,i) = obj.dpfr;
                y(:,i) = outlet;
                
                % Error analysis
                D_cp = cpdensity * 0.001; % error in carrier phase density
                D_dp = obj.dpdensity * 0.001; % error in dispersed phase denisty
                D_m = 0.01; % error in mass
                t = obj.time_collect_elapsed; % save space
                D_t = 0.1/60; % error in time
                D_Qin = obj.dpfr * 0.01; % error in injection rate
               
                % For the case where the mass collected exceeds the mass we
                % expect:
                D_Q_more = abs(D_Qin .* (1 - (obj.dpdensity/cpdensity))) + ...
                    abs(D_m./(cpdensity .*t)) + ...
                    abs(D_cp * ((-mass_out ./ (cpdensity^2 * t)) + (obj.dpdensity .* obj.dpfr / (cpdensity^2)))) +...
                    abs(D_dp.*(-obj.dpfr./cpdensity)) + ...
                    abs(D_t .* (-mass_out ./ (cpdensity .* t .* t)));
                % For the case where the mass collected is less than the
                % mass we expect:
                D_Q_less = (abs(D_m./mass_out) + abs(D_dp/obj.dpdensity) + abs(D_t./t)) .* outlet;
                
                % if mass out > mass in:                   
                    % bigger formula
                % if mass out <= mass in
                    % smaller formula
               
                y_err(:,i) = ((mass_out > dp_mass_in) .* D_Q_more) + ...
                    ((mass_out <= dp_mass_in) .* D_Q_less);
                
                fprintf('Max error in collection rate = +/-%.1f uL/min\n',...
                    max(y_err(:)))
            end

            % Choose either error bars or no error bars.
            h = plot(x, y, 'o');
            % h = errorbar(x, y, y_err, '.')
            set(h, 'MarkerSize',8);
            legend(name);
        end        
        
        %% Dependent properties
        % Continuous phase volumetric injection rate
        function cpfr = get.cpfr(obj)
            cpfr = obj.tfr ./ (obj.dpcp + 1);
        end
        
        % Dispersed phase volumetric injection rate
        function dpfr = get.dpfr(obj)
            dpfr = (obj.tfr * obj.dpcp) / (obj.dpcp + 1);
        end
        
        % Volume of DP collected at outlet (assuming ONLY DP passed through
        % to the outlet
        function dp_vol_out = get.dp_vol_out(obj)
            dp_vol_out = ((obj.mass_final - obj.mass_initial) ./...
                obj.dpdensity) * 1000;
        end
        
        % Flow rate of DP through the outlet, from dp_vol_out.
        function dp_fr_out = get.dp_fr_out(obj)
            dp_fr_out = dp_vol_out / obj.time_collect_elapsed;
        end
        
        %% Volume of each phase required to finish experiments
        % Useful to check if the Syrris bottles need refilling.
        % [dpvol, cpvol] in mL
        function vol = volreq(obj)
            vol(1) = sum(obj.dpfr .* (obj.time_stab + obj.time_collect) ...
                .* isnan(obj.mass_initial)) / 1000;
            vol(2) = sum(obj.dpfr .* (obj.time_stab + obj.time_collect) ...
                .* isnan(obj.mass_initial)) / 1000;
        end 
    end
end

